/*
    Agent - Event Scheduling  (AES) Library 
    Copyright (C) 2017  Marko Djogatovic

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

// Zaglavlje aes_dist.h
#ifndef __AES_DIST_H__
#define __AES_DIST_H__

#include "aes.h"
#include <cstdint>
#include <cmath>

// Ukljucujemo prostor imena std;
using namespace std;

namespace aes {
  // Klasa raspodela
  class raspodela {
    // Seme generatora slucajnih brojeva (GSB)
    uint32_t s;
    // Multiplikator
    uint32_t a;
    // Modulus
    uint32_t m;
  public:
    // Konstruktor
    raspodela(const uint32_t s):s(s) {
      // Postavljamo vrednost multiplikatora
      a = 16807;
      // Postavljamo vrednost modulusa
      m = 2147483647;
    }
    // Lehmerov multiplikativni kongruentni generator
    double operator() () {
      s = static_cast<uint64_t>(a*s)%m;
      return s/(1.0*m);
    }
    // Uniformna raspodela
    double unif(const double l,const double u) {
      return l+(u-l)*(*this)();
    }
    // Eksponencijalna raspodela
    double expo(const double mi) {
      return -mi*log((*this)());
    }
    // Normalna raspodela
    double norm(const double mi,const double sd) {
      double z = 0.0;
      for(int i=0; i<12; i++)
        z += (*this)();
      return mi+(z-6)*sd;
    }
  };  
}
#endif // __AES_DIST_H__