// Datoteka zad3.cpp
#include "aes.h"
#include "aes_proc.h"
#include "aes_dist.h"
#include <iostream>

// Ukljucujemo prostore imena esa i std
using namespace aes;
using namespace std;

  // Objekat reda cekanja
red salt_red;
// Salteri
resurs salteri(4);
// Objekat simulacije
simulacija simu;
raspodela rn1(11615), rn2(36517);
// Srednje vreme dolaska entiteta
const double SRVRDOL = 100;
// Srednjevreme opsluge entiteta
const double SRVROPS = 200;

// Klasa korisnika
class korisnik:public agent {
  // Brojac korisnika
  static size_t _idc;    
  // Redni broj korisnika
  size_t _id;    
public:
  // Konstruktor
  korisnik():agent() { _id = ++_idc; }
public:
  // Vraca id korisnika
  size_t vrati_id() { return _id; } 
}; 
size_t korisnik::_idc = 0;

// Klasa dogadjaja dolazak korisnika
class dolazak_korisnika:public dogadjaj {
public:
  // Konstruktor
  dolazak_korisnika():dogadjaj(1) {}
  // Akcija
  void akcija();
  // Pomocna funkcija za pravljenje dogadjaja 
  static dolazak_korisnika* napravi(korisnik* kor,vreme vn) {
    dolazak_korisnika* dk = new dolazak_korisnika;
    dk->postavi_agenta(0,kor);
    dk->postavi_vreme(simu.vrati_vreme()+vn);
    return dk;
  }
};

// Klasa dogadjaja odlazak korisnika
class odlazak_korisnika:public dogadjaj {
public:
  // Konstruktor
  odlazak_korisnika():dogadjaj(1) {}
  // Akcija dogadjaja
  void akcija();
  // Pomocna funkcija za pravljenje dogadjaja
  static odlazak_korisnika* napravi(korisnik* kor,vreme vn) {
    odlazak_korisnika* ok = new odlazak_korisnika;
    ok->postavi_agenta(0,kor);
    ok->postavi_vreme(simu.vrati_vreme()+vn);
    return ok;
  }
};

// Klasa dogadjaja zavrsetka simulacije
class kraj: public dogadjaj {
public:
  // Konstruktor dogadjaja kraj
  kraj():dogadjaj(0) {}
public:
  // Akcija dogadjaja
  void akcija() { simu.zaustavi(); }
  // Pomocna funkcija za pravljenje dogadjaja
  static kraj* napravi(vreme vn) {
    kraj* k = new kraj;
    k->postavi_vreme(simu.vrati_vreme()+vn);
    return k;
  }
};

// Akcija
void dolazak_korisnika::akcija() {
  korisnik *kor,*prvi_kor,*novi_kor;
  // Korisnik koji dolazi u postu
  kor = dynamic_cast<korisnik*>(vrati_agenta(0));
  #ifdef STAMPA
  cout << "ured" << "," << kor->vrati_id() << "," << simu.vrati_vreme() << endl;
  #endif
  // Postavi klijenta u red cekanja
  salt_red.smesti(kor);
  // Ukoliko je salter raspoloziv
  if(salteri.raspoloziv()) {
    // Izvadi prvog klijenta iz reda
    prvi_kor = dynamic_cast<korisnik*>(salt_red.prednji());
    // Prvi korisnik izlazi iz reda
    salt_red.vadi();
    #ifdef STAMPA
    cout << "izred" << "," << kor->vrati_id() << "," << simu.vrati_vreme() << endl;
    #endif
    // Zauzmi jedno mesto u salteru
    salteri.zauzmi();
    #ifdef STAMPA
    cout << "usal" << "," << kor->vrati_id() << "," << simu.vrati_vreme() << endl;
    #endif
    // Rasporedi klijenta za kraj opsluge 
    simu.rasporedi(odlazak_korisnika::napravi(prvi_kor,rn2.expo(SRVROPS)));
  }
  // Stvaramo narednog korisnika
  novi_kor = new korisnik;    
  // Postavljamo vreme narednog dolaska
  simu.rasporedi(dolazak_korisnika::napravi(novi_kor,rn1.expo(SRVRDOL)));
}

// Akcija dogadjaja
void odlazak_korisnika::akcija() {
  korisnik *kor, *prvi_kor;
  // Korisnik koji odlazi
  kor = dynamic_cast<korisnik*>(vrati_agenta(0));
  // Vrati salter
  salteri.oslobodi();
  #ifdef STAMPA
  cout << "izsal" << "," << kor->vrati_id() << "," << simu.vrati_vreme() << endl;  
  #endif
  // Klijent odlazi iz poste - unistavamo tekuceg klijenta
  delete kor;
  // Ukoliko red nije prazan
  if(salt_red.vrati_velicinu()>0) {
    // Izvadi prvog korisnika iz reda
    prvi_kor = dynamic_cast<korisnik*>(salt_red.prednji());
    salt_red.vadi();
    #ifdef STAMPA
    cout << "izred" << "," << prvi_kor->vrati_id() << "," << simu.vrati_vreme() << endl;  
    #endif
    // Zauzmi jedno mesto u salteru
    salteri.zauzmi();
    #ifdef STAMPA
    cout << "usal" << "," << prvi_kor->vrati_id() << "," << simu.vrati_vreme() << endl;  
    #endif
    // Rasporedi klijenta za kraj opsluge 
    simu.rasporedi(odlazak_korisnika::napravi(prvi_kor,rn2.expo(SRVROPS)));
  }
}

int main() {
  // Stvaramo prvog korisnika
  korisnik *kor = new korisnik;
  // Postavljamo prvi dogadja u listu sto je odgovara dolasku prvog klijenta
  simu.rasporedi(dolazak_korisnika::napravi(kor,rn1.expo(SRVRDOL)));
  // Rasporedi dogadjaj zavrsetka simulacije
  simu.rasporedi(kraj::napravi(43200));
  // Izvrsavamo simulaciju
  simu.izvrsi();
  #ifdef STAMPA
  // Stampamo vreme simulacije
  cout << "vsim" << "," << -1 << "," << simu.vrati_vreme() << endl; 
  #endif
  return 0;
}